@@ -9,22 +9,21 @@ day when using the directory structure found on the LF Radio Lab's data server.
 """
 
 import os
-from datetime import timedelta
-from datetime import datetime
+from datetime import datetime, timedelta
 
-import numpy as np
 import matplotlib.pyplot as plt
-from scipy.io import loadmat
+import numpy as np
 from matplotlib import rcParams
 from matplotlib.dates import DateFormatter
+from scipy.io import loadmat
 
+import lf.calibration as cal
 import lf.txrx as txrx
 import lf.utils
-import lf.calibration as cal
 
 
 class LFData(object):
-    """ Manage VLF data for a single transmit-receive path.
+    """Manage VLF data for a single transmit-receive path.
 
     Attributes
     ----------
@@ -80,7 +79,7 @@ class LFData(object):
     """
 
     def __init__(self, mat_files=None, data_dicts=None):
-        """ Load in either .mat files or data dictionaries for a single path
+        """Load in either .mat files or data dictionaries for a single path
 
         Parameters
         ----------
@@ -92,15 +91,13 @@ class LFData(object):
         self.rotated = False
         if mat_files is not None:
             if data_dicts is not None:
-                print(
-                    "Both mat_files and data_dicts reported, using mat_files."
-                )
+                print("Both mat_files and data_dicts reported, using mat_files.")
             self.load_mats(mat_files)
         elif data_dicts is not None:
             self.load_dicts(data_dicts)
 
     def load_mats(self, mat_files):
-        """ Load .mat files into the LFData object
+        """Load .mat files into the LFData object
 
         Parameters
         ----------
@@ -122,7 +119,7 @@ class LFData(object):
         self.combine_data(data)
 
     def load_dicts(self, data_dicts):
-        """ Load data dictionaries into LFData object
+        """Load data dictionaries into LFData object
 
         Parameters
         ----------
@@ -143,7 +140,7 @@ class LFData(object):
         self.combine_data(data)
 
     def combine_data(self, data_list):
-        """ Combine amplitude and phase data into a single data structure
+        """Combine amplitude and phase data into a single data structure
 
         Parameters
         ----------
@@ -181,30 +178,20 @@ class LFData(object):
         for key in matched_keys:
             try:
                 values = [
-                    data_list[ch][entry][key]
-                    for ch in ["NS", "EW"]
-                    for entry in [0, 1]
+                    data_list[ch][entry][key] for ch in ["NS", "EW"] for entry in [0, 1]
                 ]
                 if type(values[0]) == np.float64:
-                    if not all(
-                        np.isclose(v, values[0], rtol=0.05) for v in values
-                    ):
+                    if not all(np.isclose(v, values[0], rtol=0.05) for v in values):
                         raise ValueError(f"Data differs in {key}")
                 elif not all(v == values[0] for v in values):
                     raise ValueError(f"Data differs in {key}")
             except KeyError:
                 print(f"Unable to verify {key} values due to missing key")
         try:
-            if not (
-                data_list["NS"][0]["is_amp"]
-                == data_list["EW"][0]["is_amp"]
-                == 1
-            ):
+            if not (data_list["NS"][0]["is_amp"] == data_list["EW"][0]["is_amp"] == 1):
                 raise ValueError("Input data is not in the correct order")
             elif not (
-                data_list["NS"][1]["is_amp"]
-                == data_list["EW"][1]["is_amp"]
-                == 0
+                data_list["NS"][1]["is_amp"] == data_list["EW"][1]["is_amp"] == 0
             ):
                 raise ValueError("Input data is not in the correct order")
         except KeyError:
@@ -239,7 +226,7 @@ class LFData(object):
                 setattr(self, key, value)
 
     def calibrate(self, cal_table=None, cal_table_path=None, cal_dir=None):
-        """ Calibrate the data using a calibration table
+        """Calibrate the data using a calibration table
 
         Parameters
         ----------
@@ -267,12 +254,10 @@ class LFData(object):
                 raise RuntimeError(
                     "Must Provide either Calibration table or path to both cal_table_path and cal_dir"
                 )
-        self.data = cal_table.cal_data(
-            self.data, self.rx, self.tx, self.start_time
-        )
+        self.data = cal_table.cal_data(self.data, self.rx, self.tx, self.start_time)
 
     def rotate_data(self, correction_val=0.0):
-        """ Rotate data to be in az and radial components
+        """Rotate data to be in az and radial components
 
         Parameters
         ----------
@@ -281,26 +266,39 @@ class LFData(object):
 
         Returns
         -------
-        None
+        dict:
+            Dictionary containing VLF data
 
         """
-        amp_ns, phase_ns = self.data["NS"]
-        amp_ew, phase_ew = self.data["EW"]
-        b_ns = amp_ns * np.exp(1j * np.deg2rad(phase_ns))
-        b_ew = amp_ew * np.exp(1j * np.deg2rad(phase_ew))
-        az = lf.utils.get_azimuth(self.rx, self.call_sign) - correction_val
-        b_r, b_az = lf.utils.rot_az(az).dot([b_ns, -b_ew])
-        amp_r = np.abs(b_r)
-        phase_r = np.rad2deg(np.angle(b_r))
-        amp_az = np.abs(b_az)
-        phase_az = np.rad2deg(np.angle(b_az))
-        self.data["R"] = np.array([amp_r, phase_r])
-        self.data["Az"] = np.array([amp_az, phase_az])
+        self.data["R"], self.data["Az"] = rotate_vectors(
+            self.data["NS"], self.data["EW"], self.rx, self.tx, correction_val
+        )
         self.rotated = True
         return self.data
 
+    def rotate_polar(self):
+        """Rotate data to be in polarization ellipse format
+
+        Returns
+        -------
+        dict:
+            Dictionary containing VLF data
+
+        """
+        if ~(self.rotated):
+            self.rotate_data()
+        (
+            self.data["Major"],
+            self.data["Minor"],
+            self.data["Tilt"],
+            self.data["Start"],
+            self.data["Chi"],
+        ) = rotate_vectors_ellipse(self.data["R"], self.data["Az"])
+        self.polar = True
+        return self.data
+
     def trim(self, start, duration):
-        """ Cut out data that is not needed
+        """Cut out data that is not needed
 
         Parameters
         ----------
@@ -327,7 +325,7 @@ class LFData(object):
         self.start_time = start
 
     def plot(self):
-        """ Plot the data
+        """Plot the data
 
         Returns
         -------
@@ -341,17 +339,13 @@ class LFData(object):
         ]
         if "R" in self.data.keys() and "Az" in self.data.keys():
             fig, axes = plt.subplots(2, 4, figsize=(18, 8), sharex=True)
-            axes[1, 0].plot(
-                axis_time[::40], 20 * np.log10(self.data["Az"][0][::40])
-            )
+            axes[1, 0].plot(axis_time[::40], 20 * np.log10(self.data["Az"][0][::40]))
             axes[1, 0].set_ylabel("Amplitude [dB]")
             axes[1, 0].set_title("Az Amplitude")
             axes[1, 1].plot(axis_time[::40], self.data["Az"][1][::40])
             axes[1, 1].set_ylabel("Amplitude [dB]")
             axes[1, 1].set_title("Az Phase")
-            axes[1, 2].plot(
-                axis_time[::40], 20 * np.log10(self.data["R"][0][::40])
-            )
+            axes[1, 2].plot(axis_time[::40], 20 * np.log10(self.data["R"][0][::40]))
             axes[1, 2].set_ylabel("Amplitude [dB]")
             axes[1, 2].set_title("Radial Amplitude")
             axes[1, 3].plot(axis_time[::40], self.data["R"][1][::40])
@@ -359,17 +353,13 @@ class LFData(object):
             axes[1, 3].set_title("Radial Phase")
         else:
             fig, axes = plt.subplots(1, 4, figsize=(18, 4), sharex=True)
-        axes[0, 0].plot(
-            axis_time[::40], 20 * np.log10(self.data["NS"][0][::40])
-        )
+        axes[0, 0].plot(axis_time[::40], 20 * np.log10(self.data["NS"][0][::40]))
         axes[0, 0].set_ylabel("Amplitude [dB]")
         axes[0, 0].set_title("N/S Amplitude")
         axes[0, 1].plot(axis_time[::40], self.data["NS"][1][::40])
         axes[0, 1].set_ylabel("Amplitude [dB]")
         axes[0, 1].set_title("N/S Phase")
-        axes[0, 2].plot(
-            axis_time[::40], 20 * np.log10(self.data["EW"][0][::40])
-        )
+        axes[0, 2].plot(axis_time[::40], 20 * np.log10(self.data["EW"][0][::40]))
         axes[0, 2].set_ylabel("Amplitude [dB]")
         axes[0, 2].set_title("E/W Amplitude")
         axes[0, 3].plot(axis_time[::40], self.data["EW"][1][::40])
@@ -392,7 +382,7 @@ class LFData(object):
 
 
 def load_rx_data(mat_file, variables=None, file_check=True):
-    """ Properly format an LF AWESOME receiver's output mat file
+    """Properly format an LF AWESOME receiver's output mat file
 
     Parameters
     ----------
@@ -454,14 +444,9 @@ def load_rx_data(mat_file, variables=None, file_check=True):
         elif key in ["is_amp", "is_msk", "is_broadband"]:
             # Should be boolean
             data[key] = bool(data[key][0][0])
-        elif key in [
-            "hardware_description",
-            "station_name",
-            "call_sign",
-            "VERSION",
-        ]:
+        elif key in ["hardware_description", "station_name", "call_sign", "VERSION"]:
             # Should be strings, but are in array of ascii
-            data[key] = "".join(chr(char) for char in data[key])
+            data[key] = "".join(chr(char) for char in data[key].ravel())
     time_vals = [
         "start_year",
         "start_month",
@@ -495,7 +480,7 @@ def load_rx_data(mat_file, variables=None, file_check=True):
 
 
 def check_mat(mat_file, variables=None):
-    """ Check if a .mat file is a valid LF data mat file
+    """Check if a .mat file is a valid LF data mat file
 
     Parameters
     ----------
@@ -555,7 +540,7 @@ def check_mat(mat_file, variables=None):
 
 
 def locate_mat(data_path, date, tx, rx, resolution):
-    """ Determine the four mat_files associated with the provided Tx-Rx Path
+    """Determine the four mat_files associated with the provided Tx-Rx Path
 
     Parameters
     ----------
@@ -606,15 +591,13 @@ def locate_mat(data_path, date, tx, rx, resolution):
     ]
     for filename in filenames:
         if not os.path.exists(filename):
-            print(
-                f"Data missing from {tx}-{rx} on {date.strftime('%b %d, %Y')}"
-            )
+            print(f"Data missing from {tx}-{rx} on {date.strftime('%b %d, %Y')}")
             return None
     return filenames
 
 
 def join_days(days):
-    """ Join two consecutive instances of LFData objects
+    """Join two consecutive instances of LFData objects
 
     Parameters
     ----------
@@ -651,3 +634,120 @@ def join_days(days):
             raise RuntimeError("Data is not consistently rotated")
 
     return data
+
+
+def rotate_vectors(NS, EW, rx, tx, correction_val=0.0):
+    """Rotate a set of tuples from N/S, E/W orientation to Radial, Azimuthal orientation
+
+    Parameters
+    ----------
+    NS: tuple(np.ndarray, np.ndarray)
+        N/S amplitude, phase
+    EW: tuple(np.ndarray, np.ndarray)
+        E/W amplitude, phase
+    rx: str
+        Receiver name
+    tx: str
+        Transmitter name
+
+    Returns
+    -------
+    tuple(np.ndarray, np.ndarray)
+        ([radial amplitude, radial phase], [azimuthal amplitude, azimuthal phase])
+    """
+    amp_ns, phase_ns = NS
+    amp_ew, phase_ew = EW
+    b_ns = amp_ns * np.exp(1j * np.deg2rad(phase_ns))
+    b_ew = amp_ew * np.exp(1j * np.deg2rad(phase_ew))
+    az = lf.utils.get_azimuth(rx, tx) - correction_val
+    b_r, b_az = lf.utils.rot_az(az).dot([b_ns, -b_ew])
+    amp_r = np.abs(b_r)
+    phase_r = np.rad2deg(np.angle(b_r))
+    amp_az = np.abs(b_az)
+    phase_az = np.rad2deg(np.angle(b_az))
+    return (np.array([amp_r, phase_r]), np.array([amp_az, phase_az]))
+
+
+def rotate_vectors_ellipse(R, Az):
+    """Rotate a set of tuples from Radial/Azimuthal orientation to a polarization ellipse format as described by Gross (2018), with a major and minor axis amplitude components, tilt angle, and start phase.
+
+    Parameters:
+    ----------
+    R: tuple(np.ndarray, np.ndarray)
+        Radial amplitude, phase
+    Az: tuple(np.ndarray, np.ndarray)
+        Azimuthal amplitude, phase
+
+    Returns
+    -------
+    tuple(ndarray)
+        [major axis, minor axis, tilt angle, start phase, chi]
+
+    """
+    amp_r, phase_r = R
+    amp_az, phase_az = Az
+    B_r = amp_r * np.exp(1j * np.deg2rad(phase_r))
+    B_az = amp_az * np.exp(1j * np.deg2rad(phase_az))
+    # Calculate tilt angle
+    psi_0 = np.angle(np.divide(-B_r, B_az))
+    gamma = np.divide(amp_r, amp_az)
+    tilt_angle = (1 / 2) * np.arctan2(
+        2 * np.multiply(gamma, np.cos(psi_0)), (1 - np.square(gamma))
+    )
+    # Apply rotation
+    rot_matrix = lf.utils.rot_tilt(tilt_angle)
+    B_min, B_maj = 1j * np.zeros([2, len(B_r)])
+    for i in range(len(B_r)):
+        B_min[i], B_maj[i] = np.dot(rot_matrix[:, :, i], np.array([B_r[i], B_az[i]]))
+    start_phase = -1 * np.angle(B_maj)
+    amp_maj = np.abs(B_maj)
+    amp_min = np.abs(B_min)
+    # Calculate ellipticity
+    chi = 0.5 * np.arcsin(
+        2 * np.multiply(np.divide(gamma, (1 + np.square(gamma))), np.sin(psi_0))
+    )
+    # Flip sign of minor axis to indicate rotation direction
+    amp_min = np.multiply(np.sign(chi), amp_min)
+    return amp_maj, amp_min, tilt_angle, start_phase, chi
+
+
+def scattered_ellipse(B_amb, B_dist):
+    """
+    Calculates the polarization ellipse of the scattered field, given two
+    sets of measurements in polarization ellipse format.
+
+    Parameters
+    ------------
+    B_amb: tuple(np.ndarray)
+           Ambient major axis, minor axis, tilt angle
+    B_dist: tuple(np.ndarray)
+           Disturbed major axis, minor axis, tilt angle
+
+    Returns
+    ------------
+    B_scattered:
+           tuple(np.ndarray)
+           Scattered major axis, minor axis, tilt angle
+    """
+    maj_amb, min_amb, tilt_amb = B_amb
+    maj_dist, min_dist, tilt_dist = B_dist
+    # Use law of cosines to calculate disturbed ellipse axes
+    maj_scattered = np.square(maj_amb) + np.square(maj_dist)
+    correction = 2 * np.multiply(
+        np.multiply(maj_amb, maj_dist), np.cos(tilt_dist - tilt_amb)
+    )
+    maj_scattered = np.sqrt(maj_scattered - correction)
+    min_scattered = np.square(min_amb) + np.square(min_dist)
+    correction = 2 * np.multiply(
+        np.multiply(min_amb, min_dist), np.cos(tilt_dist - tilt_amb)
+    )
+    min_scattered = np.sqrt(min_scattered - correction)
+    # compute x and y projections of major axis to find tilt angle
+    y_maj = np.multiply(maj_dist, np.sin(tilt_dist)) - np.multiply(
+        maj_amb, np.sin(tilt_amb)
+    )
+    x_maj = np.multiply(maj_dist, np.cos(tilt_dist)) - np.multiply(
+        maj_amb, np.cos(tilt_amb)
+    )
+    tilt_scattered = np.arctan2(y_maj, x_maj)
+    return (maj_scattered, min_scattered, tilt_scattered)
